@@ -31,6 +31,8 @@
 ##' If \code{cut_type==2}, then cut randomly between the two values.
 ##' @param nboot an integer value that specifies the number of bootstrap replications.
 ##' @param seed an integer value that specifies the seed.
+##' @param args_miceRanger a list specifying additional arguments to be used to impute missing data using \code{miceRanger()}. 
+##' Only applies when \code{missing=="miceRanger"}.
 ##' @return A list containing the information of the survival forest fit.
 ##' @examples 
 ##' \donttest{
@@ -44,21 +46,22 @@
 ##'     nboot=20)
 ##' }
 survival_forest_matrix<-function(
-  time,
-  event,
-  matrix_numeric,
-  matrix_factor,
-  weights=rep(1,length(time)),
-  significance=0.05,
-  min_weights=50,
-  missing="omit",
-  test_type="univariate",
-  cut_type=0,
-  nboot=100,
-  seed=0){
+    time,
+    event,
+    matrix_numeric,
+    matrix_factor,
+    weights=rep(1,length(time)),
+    significance=0.05,
+    min_weights=50,
+    missing="omit",
+    test_type="univariate",
+    cut_type=0,
+    nboot=100,
+    seed=0,
+    args_miceRanger=NULL){
   
   # check [missing], [test_type]
-  if(!missing%in%c("majority","omit","weighted"))stop("Invalid 'missing' argument.")
+  if(!missing%in%c("majority","omit","weighted","miceRanger"))stop("Invalid 'missing' argument.")
   if(!test_type%in%c("univariate",p.adjust.methods))stop("Invalid 'test_type' argument.")
   
   event<-as.logical(event)
@@ -85,6 +88,7 @@ survival_forest_matrix<-function(
   if(is.null(colnames(matrix_numeric))&ndim_numeric>=1)colnames(matrix_numeric)<-paste0("numeric",1:ncol(matrix_numeric),sep="")
   if(is.null(colnames(matrix_factor))&ndim_factor>=1)colnames(matrix_factor)<-paste0("factor",1:ncol(matrix_factor),sep="")
   variable_names<-c(colnames(matrix_numeric),colnames(matrix_factor))
+  if(any(duplicated(variable_names)))stop("Duplicated variable names detected.")
   
   # convert matrix_factor to an integer matrix
   # factor_dictionary<-list()
@@ -100,35 +104,80 @@ survival_forest_matrix<-function(
   # }
   # matrix_factor<-matrix_factor_int
   
-  a_survival_forest<-list()
-  set.seed(seed)
-  for(boot_idx in 1:nboot){
-    cat(boot_idx)
-    shuffle<-sample(1:nind,nind,replace=T)
-    time_boot<-time[shuffle]
-    event_boot<-event[shuffle]
-    matrix_numeric_boot<-matrix_numeric[shuffle,,drop=FALSE]
-    matrix_factor_boot<-matrix_factor[shuffle,,drop=FALSE]
-    weights_boot<-weights[shuffle]
-    
-    a_survival_tree<-survival_tree_matrix(
-      time=time_boot,
-      event=event_boot,
-      matrix_numeric=matrix_numeric_boot,
-      matrix_factor=matrix_factor_boot,
-      weights=weights_boot,
-      significance=significance,
-      min_weights=min_weights,
-      missing=missing,
-      test_type=test_type,
-      cut_type=cut_type)
-    cat(" - ")
-    a_survival_forest[[boot_idx]]<-a_survival_tree
+  if(missing%in%c("majority","omit","weighted")){
+    set.seed(seed)
+    a_survival_forest<-list()
+    for(boot_idx in 1:nboot){
+      cat(boot_idx,", ")
+      shuffle<-sample(1:nind,nind,replace=TRUE)
+      time_boot<-time[shuffle]
+      event_boot<-event[shuffle]
+      matrix_numeric_boot<-matrix_numeric[shuffle,,drop=FALSE]
+      matrix_factor_boot<-matrix_factor[shuffle,,drop=FALSE]
+      weights_boot<-weights[shuffle]
+      
+      a_survival_tree<-survival_tree_matrix(
+        time=time_boot,
+        event=event_boot,
+        matrix_numeric=matrix_numeric_boot,
+        matrix_factor=matrix_factor_boot,
+        weights=weights_boot,
+        significance=significance,
+        min_weights=min_weights,
+        missing=missing,
+        test_type=test_type,
+        cut_type=cut_type)
+      a_survival_forest[[boot_idx]]<-a_survival_tree
+    }
+    return(list(
+      variable_names=variable_names,
+      ndim_numeric=ndim_numeric,
+      ndim_factor=ndim_factor,
+      survival_forest=a_survival_forest))
+  }else if(missing=="miceRanger"){
+    if(!requireNamespace("miceRanger",quietly=TRUE))  
+      stop("The 'miceRanger' package needs to be installed when missing=='miceRanger'.")
+    set.seed(seed)
+    a_survival_forest<-list()
+    df_temp<-cbind(as.data.frame(matrix_numeric),as.data.frame(matrix_factor))
+    args_miceRanger$data<-df_temp
+    a_miceRanger<-do.call(getExportedValue("miceRanger","miceRanger"),args_miceRanger)
+    list_df_imputed<-getExportedValue("miceRanger", "completeData")(a_miceRanger)
+    list_matrix_numeric<-list()
+    list_matrix_factor<-list()
+    nimpute<-length(list_df_imputed)
+    for(data_idx in 1:nimpute){
+      list_matrix_numeric[[data_idx]]<-as.matrix(mutate_all(dplyr::as_tibble(
+        list_df_imputed[[data_idx]])[,colnames(matrix_numeric)],as.numeric))
+      list_matrix_factor[[data_idx]]<-as.matrix(mutate_all(dplyr::as_tibble(
+        list_df_imputed[[data_idx]])[,colnames(matrix_factor)],as.character))
+    }
+    boot_to_data<-sample(rep(sample(1:nimpute),length.out=nboot))
+    for(boot_idx in 1:nboot){
+      cat(boot_idx,", ")
+      shuffle<-sample(1:nind,nind,replace=TRUE)
+      time_boot<-time[shuffle]
+      event_boot<-event[shuffle]
+      matrix_numeric_boot<-list_matrix_numeric[[boot_to_data[boot_idx]]][shuffle,,drop=FALSE]
+      matrix_factor_boot<-list_matrix_factor[[boot_to_data[boot_idx]]][shuffle,,drop=FALSE]
+      weights_boot<-weights[shuffle]
+      a_survival_tree<-survival_tree_matrix(
+        time=time_boot,
+        event=event_boot,
+        matrix_numeric=matrix_numeric_boot,
+        matrix_factor=matrix_factor_boot,
+        weights=weights_boot,
+        significance=significance,
+        min_weights=min_weights,
+        missing="omit",
+        test_type=test_type,
+        cut_type=cut_type)
+      a_survival_forest[[boot_idx]]<-a_survival_tree
+    }
+    return(list(
+      variable_names=variable_names,
+      ndim_numeric=ndim_numeric,
+      ndim_factor=ndim_factor,
+      survival_forest=a_survival_forest))
   }
-  
-  return(list(
-    variable_names=variable_names,
-    ndim_numeric=ndim_numeric,
-    ndim_factor=ndim_factor,
-    survival_forest=a_survival_forest))
 }
\ No newline at end of file
